/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: BufferedImage.cpp
 * Description: BufferedImage methods. (SDL)
 * =============================================
 */

#include <includes.h>
#include <BufferedImage.h>

BufferedImage::BufferedImage() {
    width = 800;
    height = 600;
    bpp = 32;
    init();
}

BufferedImage::BufferedImage(int Width, int Height, int BPP) {
    width = Width;
    height = Height;
    bpp = BPP;
    init();
}

void BufferedImage::init() {
    
    SDL_Surface * disp = SDL_GetVideoSurface();
    
    if(disp == NULL)
        fatalError("BufferedImage::init","Could not get video surface.");
    
    Uint32 rmask, gmask, bmask, amask;

    rmask = disp->format->Rmask;
    gmask = disp->format->Gmask;
    bmask = disp->format->Bmask;
    amask = disp->format->Amask;

    image.s = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, bpp, rmask, gmask, bmask, amask);
    
    if(image.s == NULL)
        fatalError("BufferedImage::init","Unable to allocate output image.");
}

BufferedImage::~BufferedImage() {
    SDL_FreeSurface(image.s);
}

void BufferedImage::drawImage(Surface screen) {
    // Draws the image on the screen:
    SDL_BlitSurface(image.s, NULL, screen.s, NULL);
}

void BufferedImage::setRGB(int x, int y, uint32_t pixel) {
    SDL_Rect rc = { x, y, 1, 1 };
    SDL_FillRect(image.s, &rc, (Uint32)pixel);
}

void BufferedImage::saveBMP(char * filename) {
    if(SDL_SaveBMP(image.s, filename) == -1)
        message("BufferedImage::saveBMP","Could not save image.");
}

uint32_t BufferedImage::getRGB(int x, int y) {
    uint32_t * pixels = (uint32_t *)image.s->pixels;
    return pixels[(y * image.s->w)+x];
}
