/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Video.cpp
 * Description: Video output functions. (SDL)
 * =============================================
 */

#include <includes.h>
#include <Video.h>

/* Video */
Surface Display;

void Video::initVideo(int width, int height, int bpp) {
    
    /* SDL Initialization */
    if(SDL_Init(SDL_INIT_EVERYTHING) < 0) {
        fatalError("Video::initVideo","Unable to start SDL.");
    }
    
    if((Display.s = SDL_SetVideoMode(width, height, bpp, SDL_SWSURFACE | SDL_DOUBLEBUF)) == NULL) {
        fatalError("Video::initVideo","Unable to start video.");
    }
}

void Video::refreshVideo() {
    SDL_Flip(Display.s);
}

void Video::delayVideo(int delay) {
    SDL_Delay(delay);
}

void Video::finishVideo() {
    SDL_FreeSurface(Display.s);
}

/* Event */
bool Video::quitEvent() {
    SDL_Event ev;
    SDL_PollEvent(&ev);
    return (ev.type == SDL_QUIT);
}

void Video::waitQuit() {
    SDL_Event ev;    
    while(ev.type != SDL_QUIT) SDL_PollEvent(&ev);    
}

uint32_t Video::toRGB(uint8_t r, uint8_t g, uint8_t b) {
    SDL_Surface * disp = SDL_GetVideoSurface();
    
    if(disp == NULL)
        fatalError("Video::toRGB","Could not get video surface.");
    
    return (uint32_t) SDL_MapRGB(disp->format, r, g, b);
}
