/* =============================================
 * Open Core RayTracer - Animation component
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Animation.h
 * Description: Animation (Uniform B-Spline Curves)
 * =============================================
 */

#ifndef _ANIMATION_
#define _ANIMATION_

#include <includes.h>
#include <geometry/Vector3D.h>
#include <geometry/Point3D.h>

class Animation {
  
    public:
        Animation();
        Animation(double startTimems, double endTimems);
        ~Animation();
    
        void setStart(double startTimems);
        void setEnd(double endTimems);
    
        void addControlPoint(Point3D * point);
        int getNumberOfControlPoints();
    
        bool animate(double currentTimems);
        Point3D * getCurrentPosition();
        Vector3D * getCurrentUp();
        Vector3D * getCurrentDirection();
        Vector3D * getCurrentRight();
        Vector3D * getCurrentAcceleration();
    
    private:
        double startTime;
        double endTime;
    
        Point3D currentPosition;
        Vector3D currentUp, currentDirection, currentRight;
    
        Vector3D position, velocity, acceleration;
    
        std::vector<Point3D *> controlPoints;
    
        /* Curve */
        void q(double u);
        void ql(double u);
        void qll(double u);
    
        /* Blending functions */
        double R0(double t) {
            return t*t*t/6.0;
        }
        
        double R0l(double t) {
            return t*t/2.0;
        }
        
        double R0ll(double t) {
            return t;
        }
        
        double R1(double t) {
            return (3.0*t*(-t*t + t + 1.0) + 1.0)/6.0;
        }
        
        double R1l(double t) {
            return -(t - 1.0)*(3.0*t + 1.0)/2.0;
        }
        
        double R1ll(double t) {
            return 1.0 - 3.0*t;
        }
        
        double R2(double t) {
            return (3.0*(t - 2.0)*t*t + 4.0)/6.0;
        }
        
        double R2l(double t) {
            return t*(3.0*t/2.0 - 2.0);
        }
        
        double R2ll(double t) {
            return 3.0*t - 2.0;
        }
        
        double R3(double t) {
            return t*(-0.5 + t*(0.5 - t/6.0)) + 1.0/6.0;
        }
        
        double R3l(double t) {
            return (1.0 - t/2.0)*t - 0.5;
        }
        
        double R3ll(double t) {
            return 1.0 - t;
        }
};

#endif
