/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Ray.cpp
 * Description: Ray methods.
 * =============================================
 */

#include <iterator>
#include <engine/Ray.h>
#include <geometry/Point3D.h>
#include <geometry/Vector3D.h>
#ifdef VERSOR
#include <vsr/vsr_cga3D_op.h>
#include <vsr/vsr_cga3D_funcs.h>
using namespace vsr::cga3D;
#endif

Ray::Ray(Point3D const& Origin, Vector3D const& Direction, float Energy) {
#ifdef GAALOP
    float Ox = Origin.getX();
    float Oy = Origin.getY();
    float Oz = Origin.getZ();
    float Lx = Origin.getX() + Direction.getX();
    float Ly = Origin.getY() + Direction.getY();
    float Lz = Origin.getZ() + Direction.getZ();
#pragma gpc begin
#pragma clucalc begin
    O = VecN3(Ox, Oy, Oz);
    L = VecN3(Lx, Ly, Lz);
    ?R = *(O ^ L ^ einf);
#pragma clucalc end
    ray = mv_to_array(R, e1^e2, e1^e3, e1^einf, e2^e3, e2^einf, e3^einf);
#pragma gpc end
#elif VERSOR
    ray = vsr::Fl::line(Origin, static_cast<Point3D>(Direction)).dual();
#else
    origin = Origin;
    direction = Direction;
#endif
    energy = Energy;
    direction = Direction; // TODO: implement in GA
}

float Ray::getEnergy() const { return energy; }
void Ray::setEnergy(float Energy) { energy = Energy; }

Ray::~Ray() {}
