/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Ray.h
 * Description: Ray class definition.
 *     Ray represents a ray from the camera.
 * =============================================
 */

#ifndef _RAY_
#define _RAY_

#include <geometry/Point3D.h>
#include <geometry/Vector3D.h>
#include <objects/Camera.h>
#ifdef VERSOR
#include <vsr/vsr_cga3D_op.h>
#include <vsr/vsr_cga3D_funcs.h>
#endif

class Ray {

    public:
        Ray(Point3D const& Origin, Vector3D const& Direction, float Energy = 1.0f);
        ~Ray();

#if defined(GAALOP) || defined(VERSOR)
        Camera * camera;
        Vector3D direction; //TODO: implement in GA
#endif
#ifdef GAALOP
        float ray[6];
#elif VERSOR
        vsr::cga3D::DualLine ray;
#else
        Point3D origin; //the ray origin
        Vector3D direction; //the ray direction
#endif

        float getEnergy() const;
        void setEnergy(float Energy);

    private:
        float energy; //the ray energy

};

#endif
