/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: RayShooter.cpp
 * Description: RayShooter methods.
 * =============================================
 */

#include <includes.h>
#include <engine/RayShooter.h>
#include <geometry/Util.h>
#include <geometry/Vector3D.h>
#include <objects/Camera.h>
#include <engine/Ray.h>

Dimension::Dimension() {
    width = 800;
    height = 600;
}

Dimension::Dimension(int Width, int Height) {
    width = Width;
    height = Height;
}

RayShooter::RayShooter(Camera * Camera, Dimension image) {
    eye = *(Camera->eye);

    //projection direction
    Vector3D * zdir = new Vector3D(*(Camera->eye), *(Camera->lookat));
    zdir->normalize();

    //projection plane x and y directions
    xdir = Util::cross_product(*zdir, *(Camera->viewup));
    xdir->normalize();

    ydir = Util::cross_product(*xdir, *zdir);
    ydir->normalize();

    //calculating the width and height of the projection plane
    float height = 2 * Camera->getZVP() * (float) tan(toRadians(Camera->getFOV()) / 2.0f);
    float width = ((float) image.width / (float) image.height) * height;

    //the x and y direction vectors will have the size of one pixel
    xdir->multiply(width / image.width);
    ydir->multiply(height / image.height);

    //creating the first ray (top left of the projection plane)
    fray = new Vector3D(*(Camera->eye), *(Camera->lookat));
    fray->normalize();
    fray->multiply(Camera->getZVP());

    fray->setX(fray->getX() + (image.height / 2.0f) * ydir->getX() - (image.width / 2.0f) * xdir->getX());
    fray->setY(fray->getY() + (image.height / 2.0f) * ydir->getY() - (image.width / 2.0f) * xdir->getY());
    fray->setZ(fray->getZ() + (image.height / 2.0f) * ydir->getZ() - (image.width / 2.0f) * xdir->getZ());

    delete zdir;

#if defined(GAALOP) || defined(VERSOR)
    // save Camera Pointer for getRay
    camera = Camera;
#endif
}

RayShooter::~RayShooter() {
    delete xdir;
    delete ydir;
    delete fray;
}

Ray RayShooter::getRay(float x, float y) {
    //calculating the ray direction based on the initial ray
    Vector3D raydir(fray->getX() + x * xdir->getX() - y * ydir->getX(), fray->getY() + x * xdir->getY() - y * ydir->getY(), fray->getZ() + x * xdir->getZ() - y * ydir->getZ());
    raydir.normalize();
    Ray ret(eye, raydir);
#if defined(GAALOP) || defined(VERSOR)
    ret.camera = camera;
#endif
    return ret;
}
