/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: RayTracing.h
 * Description: RayTracing class definition.
 *     RayTracing generates an image from a
 *     scene. It uses the ray tracing algorithm.
 * =============================================
 */

#ifndef _RAYTRACING_
#define _RAYTRACING_

#include <BufferedImage.h>
#include <geometry/Point3D.h>
#include <geometry/Util.h>
#include <geometry/Vector3D.h>
#include <objects/AbstractObject.h>
#include <objects/Color.h>
#include <objects/Light.h>
#include <objects/Material.h>
#include <objects/Scene.h>
#include <engine/Ray.h>
#include <engine/RayShooter.h>

#define RAYTRACING_MIN_ENERGY 0.001f
#define RAYTRACING_EPSILON 0.001f

class RayTracing {

    public:
        RayTracing();
        ~RayTracing();

        BufferedImage * execute(Scene const& scene, Dimension imagesize);
        BufferedImage * executeAntialiasing(Scene const& scene, Dimension imagesize);
        Color trace(Scene const& scene, Ray const& ray, int step);

    private:
        Color phongShadow(AbstractObject * object, Scene const& scene, Vector3D const& normal, Vector3D const& refldir, Point3D const& intersection);
        Color phong(AbstractObject * object, Scene const& scene, Vector3D normal, Vector3D reflectionray, Point3D intersection);

        int maxsteps;
};

#endif
