/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Util.cpp
 * Description: Util methods.
 * =============================================
 */

#include <includes.h>
#include <geometry/Util.h>
#include <geometry/Vector3D.h>
#include <objects/Color.h>

float Util::scalar_product(Vector3D a, Vector3D b) {
    return a.getX() * b.getX() + a.getY() * b.getY() + a.getZ() * b.getZ();
}

Vector3D * Util::cross_product(Vector3D a, Vector3D b) {
    return new Vector3D(a.getY() * b.getZ() - a.getZ() * b.getY(),
                        a.getZ() * b.getX() - a.getX() * b.getZ(),
                        a.getX() * b.getY() - b.getX() * a.getY());
}

Color Util::gammacorrect(Color color) {
    Color newcolor;
    
    newcolor.setRed(max(min(color.getRed() / 255.0f, 1.0f), 0.0f));
    newcolor.setGreen(max(min(color.getGreen() / 255.0f, 1.0f), 0.0f));
    newcolor.setBlue(max(min(color.getBlue() / 255.0f, 1.0f), 0.0f));

    newcolor.setRed((float) exp(log(newcolor.getRed()) / UTIL_GAMMA));
    newcolor.setGreen((float) exp(log(newcolor.getGreen()) / UTIL_GAMMA));
    newcolor.setBlue((float) exp(log(newcolor.getBlue()) / UTIL_GAMMA));

    newcolor.setRed(newcolor.getRed() * 255 + 0.5f);
    newcolor.setGreen(newcolor.getGreen() * 255 + 0.5f);
    newcolor.setBlue(newcolor.getBlue() * 255 + 0.5f);

    return newcolor;
}