/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Vector3D.cpp
 * Description: Vector3D methods.
 * =============================================
 */

#include <includes.h>
#include <geometry/Vector3D.h>
#include <geometry/Point3D.h>
#include <stdexcept>

Vector3D::Vector3D() : Point3D() {
}

Vector3D::~Vector3D() {
}

Vector3D::Vector3D(float X, float Y, float Z) : Point3D(X, Y, Z) {
}

Vector3D::Vector3D(Point3D a, Point3D b) {
    x = b.getX() - a.getX();
    y = b.getY() - a.getY();
    z = b.getZ() - a.getZ();
}

void Vector3D::normalize() {
    float nor = norm();
    if(nor != 0.0f) {
        x = x / nor;
        y = y / nor;
        z = z / nor;
    } else {
        x = 0.0f;
        y = 0.0f;
        z = 0.0f;
    }
}

void Vector3D::multiply(float scalar) {
    x = x * scalar;
    y = y * scalar;
    z = z * scalar;
}

float Vector3D::norm() {
    return (float) sqrt(x * x + y * y + z * z);
}

void Vector3D::sum(Vector3D a) {
    x += a.getX();
    y += a.getY();
    z += a.getZ();
}
