/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Parser.h
 * Description: Parser class definition.
 *     This is a simple recursive parser.
 * =============================================
 */

#ifndef _PARSER_
#define _PARSER_

class Parser {

    public:
        Parser(char * buffer, int size);
        ~Parser();
    
        void rewind();

        void getChar();
        void match(char c);
        void matchNoSkip(char c);

        void skipWhite();
        void skipUntil(char c);
        void skipUntilEnd();

        char * getItem(); /* Get Token */
        char * getString(); /* Get String */
        char * getNum(); /* Get Integer in String form */
        char * getNumF(); /* Get Float in String form */
        int getNumInt(); /* Get Integer as an Integer */
        float getNumFloat(); /* Get Float as a Float */

        bool isInt(char * s);
        bool isFloat(char * s);
        bool isEnd();

        void strToUpper(char * s);
    
        char getLook();

    private:
        char look;
        int currPos;
        int currLine;
        char * Buffer;
        int bufferSize;

        void message(const char * s);
        void warning(const char * s);
        void error(const char * s);
        void abort(const char * s);
        void expected(const char * s);
        void halt();

        bool isAlpha(char c);
        bool isDigit(char c);
        bool isAlNum(char c);
        bool isWhite(char c);
        char upCase(char c);
        
        void skipComment();

};

#endif
