/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: AbstractObject.h
 * Description: AbstractObject class definition.
 *     This is an abstract class representing
 *     an object that can be rendered.
 * =============================================
 */

#ifndef _ABSTRACT_OBJECT_
#define _ABSTRACT_OBJECT_

#include <engine/Ray.h>
#include <geometry/Point3D.h>
#include <geometry/Vector3D.h>
#include <objects/Material.h>
#include <animation/Animation.h>

class Intersection;

class AbstractObject {

    public:
        AbstractObject();
        virtual ~AbstractObject();
        void animate(double currentTimems);

        virtual
#if defined(GAALOP) || defined(VERSOR)
        Intersection
#else
        float
#endif
        intersect(Ray const& ray) = 0;
        virtual Vector3D normal(Point3D intersection) = 0;

        Material * material;

        Animation * animation;

        Point3D position;
};

#endif
