/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Camera.cpp
 * Description: Camera methods.
 * =============================================
 */

#include <iostream>
#include <objects/Camera.h>
#include <geometry/Point3D.h>
#include <geometry/Vector3D.h>

Camera::Camera() {
    eye = new Point3D(0, 0, 0);
    lookat = new Point3D(0, 0, -1);
    viewup = new Vector3D(0, 1, 0);
    fov = 45;
    zvp = 1;
    updateCam();
}

Camera::~Camera() {
    delete eye;
    delete lookat;
    delete viewup;
}
void Camera::updateCam()
{
#if defined(GAALOP) || defined(VERSOR)
    Vector3D N = Vector3D(*eye, *lookat);
    N.normalize();
    float Nx = N.getX();
    float Ny = N.getY();
    float Nz = N.getZ();
    float d = Nx * eye->getX() + Ny * eye->getY() + Nz * eye->getZ();
#endif

#ifdef GAALOP
#pragma gpc begin
#pragma clucalc begin
    ?Plane = Nx * e1 + Ny * e2 + Nz * e3 + d * einf;
#pragma clucalc end
    visibility_plane = mv_to_array(Plane, e1, e2, e3, einf);
#pragma gpc end
#elif VERSOR
    visibility_plane = vsr::Fl::plane(Nx, Ny, Nz, d);
#endif
}


float Camera::getFOV() { return fov; }
float Camera::getZVP() { return zvp; }

void Camera::setFOV(float FOV) { fov = FOV; }
void Camera::setZVP(float ZVP) { zvp = ZVP; }

