/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Camera.h
 * Description: Camera class definition.
 *     Camera represents the camera properties,
 *     position and orientation.
 * =============================================
 */

#ifndef _CAMERA_
#define _CAMERA_

#include <geometry/Point3D.h>
#include <geometry/Vector3D.h>
#ifdef VERSOR
#include <vsr/vsr_cga3D_op.h>
#include <vsr/vsr_cga3D_funcs.h>
#endif

class Camera {

    public:
        Camera();
        ~Camera();

        Point3D * eye; //camera position
        Point3D * lookat; //position that the camera is looking at
        Vector3D * viewup; //the view-up vector

        float getFOV();
        float getZVP();

        void setFOV(float FOV);
        void setZVP(float ZVP);

#ifdef GAALOP
        float visibility_plane[4];
#elif VERSOR
        vsr::cga3D::DualPlane visibility_plane;
#endif
        void updateCam();

private:
        float fov; //field of view angle (in degrees)
        float zvp; //plane distance from the camera position
};

#endif
