#ifdef GAALOP
/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Sphere.cpp
 * Description: Sphere methods.
 * =============================================
 */

#include <includes.h>
#include <iostream>
#include <engine/Ray.h>
#include <objects/Camera.h>
#include <geometry/Point3D.h>
#include <geometry/Vector3D.h>
#include <objects/Fractal.h>

Fractal::Fractal(Point3D Center, float Tolerance) {
    position = Point3D(Center.getX(), Center.getY(), Center.getZ());
    tolerance = Tolerance;
}

void Fractal::update() {

}

Fractal::~Fractal() {
}

Intersection Fractal::intersect(Ray const& ray) {
    Point3D * cam_center = ray.camera->eye;
    float CamX = cam_center->getX();
    float CamY = cam_center->getY();
    float CamZ = cam_center->getZ();
    float DirX = ray.direction.getX();
    float DirY = ray.direction.getY();
    float DirZ = ray.direction.getZ();
    float DirNorm = ray.direction.norm();
    DirX = DirX/DirNorm;
    DirY = DirY/DirNorm;
    DirZ = DirZ/DirNorm;
    //std::cout << std::endl << "Cam = (" << CamX << "," << CamY << "," << CamZ << ")" << std::endl;
    //std::cout << "Dir = (" << DirX << "," << DirY << "," << DirZ << ")" << std::endl;
#pragma gpc begin
#pragma clucalc begin
    Cam = CamX * e1 + CamY * e2 + CamZ * e3;
    ?u = DirX * e1 + DirY * e2 + DirZ * e3;
    ?x = Cam;
    ?x_old = Cam;
#pragma clucalc end
    for (int i = 0; i < 100; i++)
    {
#pragma clucalc begin
        ?d_min = (abs(x) / 4 * abs(u * (e3^e1) * x_old)) * log(abs(x)); // 1 is real axis??
        //?d_min = (abs(x) / 4 * abs(x_old)) * log(abs(x)); // 1 is real axis??
#pragma clucalc end
        float d_min_scalar = mv_get_bladecoeff(d_min, 1);
        if (d_min_scalar != d_min_scalar)
            break;
        //std::cout << i << ": d = " << d_min_scalar << std::endl;
        if (d_min_scalar < 1.0)
        {
            float ix = mv_get_bladecoeff(x, e1);
            float iy = mv_get_bladecoeff(x, e2);
            float iz = mv_get_bladecoeff(x, e3);
#pragma clucalc begin
            // convert to 5D
            ?five_d = VecN3(ix,iy,iz);
#pragma clucalc end
            Intersection inter;
            inter.valid = true;
            inter.obj = this;
            inter.p = mv_to_array(five_d, e1, e2, e3, einf, e0);
            return inter;
        }
#pragma clucalc begin
        ?x_new = x + d_min_scalar * u;
#pragma clucalc end
        //std::cout << "new_x = (" << x_new[0] << "," << x_new[1] << "," << x_new[2] << ")" << std::endl;
        std::copy(x,x+5,x_old);
        std::copy(x_new,x_new+5,x);
    }
#pragma gpc end
    return Intersection();
}

Vector3D Fractal::normal(Point3D intersection) {
    return Vector3D(1,1,1);
}

#endif
