/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Material.cpp
 * Description: Material and Component methods.
 * =============================================
 */

#include <objects/Material.h>

Component::Component() {
    red = 0.0f;
    green = 0.0f;
    blue = 0.0f;
}

Component::Component(float Red, float Green, float Blue) {
    red = Red;
    green = Green;
    blue = Blue;
}

float Component::getRed() { return red; }
float Component::getGreen() { return green; }
float Component::getBlue() { return blue; }

void Component::setRed(float Red) { red = Red; }
void Component::setGreen(float Green) { green = Green; }
void Component::setBlue(float Blue) { blue = Blue; }

Material::Material() {
    ambient = new Component(0.5f, 0.5f, 0.5f);
    diffuse = new Component(0.5f, 0.5f, 0.5f);
    specular = new Component(0.5f, 0.5f, 0.5f);
    reflt = 1;
    ns = 1;
    transp = 0;   
}

Material::~Material() {
    delete ambient;
    delete diffuse;
    delete specular;
}

void Material::setAmbient(float red, float green, float blue) {
    ambient->setRed(red);
    ambient->setGreen(green);
    ambient->setBlue(blue);
}

void Material::setDiffuse(float red, float green, float blue) {
    diffuse->setRed(red);
    diffuse->setGreen(green);
    diffuse->setBlue(blue);
}

void Material::setSpecular(float red, float green, float blue) {
    specular->setRed(red);
    specular->setGreen(green);
    specular->setBlue(blue);
}

float Material::getNS() { return ns; }
float Material::getREFLT() { return reflt; }
float Material::getTRANSP() { return transp; }

void Material::setNS(float NS) { ns = NS; }
void Material::setREFLT(float REFLT) { reflt = REFLT; }
void Material::setTRANSP(float TRANSP) { transp = TRANSP; }
