/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Material.h
 * Description: Material class definition and
 *     Component class definition.
 *     Material represents material shininess,
 *     transparency, reflection, ambient 
 *     component, diffuse component and specular
 *     component.
 * =============================================
 */

#ifndef _MATERIAL_
#define _MATERIAL_

class Component {

    public:
        Component();
        Component(float Red, float Green, float Blue);
        
        float getRed();
        float getGreen();
        float getBlue();
        
        void setRed(float Red);
        void setGreen(float Green);
        void setBlue(float Blue);

    private:
        float red;
        float green;
        float blue;

};

class Material {

    public:
    
        Material();
        ~Material();
        
        void setAmbient(float red, float green, float blue);
        void setDiffuse(float red, float green, float blue);
        void setSpecular(float red, float green, float blue);

        float getNS();
        float getREFLT();
        float getTRANSP();
        
        void setNS(float NS);
        void setREFLT(float REFLT);
        void setTRANSP(float TRANSP);

        Component * ambient; //[0,1]
        Component * diffuse; //[0,1]
        Component * specular; //[0,1]
  
    private:
        float ns; //specular coefficient [0,inf]
        float reflt; //reflection [0,1]
        float transp; // transparency [0,1]
};


#endif