/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Scene.h
 * Description: Scene class definition.
 *     Scene stores scene information (camera,
 *     lights and objects) and checks for
 *     intersection.
 * =============================================
 */

#ifndef _SCENE_
#define _SCENE_

#include <includes.h>
#include <engine/Ray.h>
#include <objects/AbstractObject.h>
#include <objects/Camera.h>
#include <objects/Light.h>
#include <objects/Intersection.h>

class Scene {

    public:
        Scene();
        ~Scene();

#if defined(GAALOP) || defined(VERSOR)
        Intersection intersect(Ray const& ray) const;
#else
        Intersection * intersect(Ray const& ray) const;
#endif
        void animate(double currentTimems);

        Color * background;
        Camera * camera;
        std::vector<AbstractObject *> objects;
        std::vector<Light *> lights;
};

#endif
