/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: Sphere.h
 * Description: Sphere class definition.
 *     Sphere is an AbstractObject.
 * =============================================
 */

#ifndef _SPHERE_
#define _SPHERE_

#include <engine/Ray.h>
#include <geometry/Point3D.h>
#include <geometry/Vector3D.h>
#include <objects/AbstractObject.h>
#include <objects/Intersection.h>
#ifdef VERSOR
#include <vsr/vsr_cga3D_op.h>
#include <vsr/vsr_cga3D_funcs.h>
#endif

class Sphere : public AbstractObject {

    public:
        Sphere(Point3D Center = Point3D(), float Radius = 1.0f);
        ~Sphere();

#if defined(GAALOP) || defined(VERSOR)
        Intersection
#else
        float
#endif
        intersect(Ray const& ray);
        Vector3D normal(Point3D intersection);

        float getRadius();
        void setRadius(float Radius);
        void update();

    private:
#ifdef GAALOP
        float sphere[5];
        float radius; // TODO: implement in GA
#elif VERSOR
        vsr::cga3D::DualSphere sphere;
        float radius; // TODO: implement in GA
#else
        float radius;
#endif
};

#endif
