/* =============================================
 * Open Core RayTracer
 * Created by Fabio de Albuquerque Dela Antonio
 * fabio914 at gmail.com
 *
 * Based on the RayTracer written by
 * Fernando Vieira Paulovich
 * (http://www.icmc.usp.br/~paulovic)
 * =============================================
 * This code is licensed under the terms of the
 * GNU General Public License (GPL).
 * =============================================
 * File: SceneLoader.h
 * Description: SceneLoader class definition.
 *     SceneLoader reads the scene description
 *     from a file and generates a scene.
 * =============================================
 */

#ifndef _SCENE_LOADER_
#define _SCENE_LOADER_

//#include <objects/Box.h>
#include <objects/Camera.h>
#include <objects/Color.h>
#include <objects/Light.h>
#include <objects/Material.h>
#include <objects/Scene.h>
#include <objects/Sphere.h>
#include <engine/RayShooter.h>
#include <animation/Animation.h>
#include <misc/Parser.h>

class SceneLoader {

    public:
        SceneLoader();
        SceneLoader(char * filename);
        ~SceneLoader();

        void parseScene();
        Scene * getScene();
        Dimension getDimension();
        bool getAntialiasing();
        char * getFilename();
        bool save();
        bool isAnimation();
        float getFramesPerSecond();
        float getAnimationTimeLength();

    private:
        Parser * p;
        Scene * s;
        Dimension d;
        char * buffer;
        char * filename;
        bool saveImage;
        bool antialiasing;
        bool animatedScene;
        float fps, totalAnimationTime;

        /* Scene properties */
        Color parseBackground();
        Dimension parseDimension();

        /* Camera */
        Camera * parseCamera();

        /* Light */
        Light * parseLight();

        /* Objects */
        Sphere * parseSphere();
        //Box * parseBox();

        /* Material */
        Material * parseMaterial();

        /* Animation */
        Animation * parseAnimation();

        /* Low level */
        void parseArray(float *x, float *y, float *z);
        float parseFloat();
        void initLoader(char const * filename);
};

#endif
